@@ -15,8 +15,6 @@ def upk(stuff):
     return stuff[0]
 
 def merge_dict(d1, d2):
-    # print "D1: ", d1
-    # print "D2: ", d2
     if d2 == None: return d1
     if type(d2) == dict:
         for k, v in d2.iteritems():
@@ -28,7 +26,6 @@ def merge_dict(d1, d2):
     else:
         print "Assertion Error: Missing .asList()?", type(d2), type([])
         sys.exit(99)
-    # print "Merged: ", d1
     return d1
 
 LPAREN = Suppress("(")
@@ -166,11 +163,14 @@ def set_column_width_if_needed(worksheet, column, width_style):
 
 def set_range_border_if_needed(worksheet, yrange, xrange, border_style):
     if border_style == None or border_style == "": return
-    if border_style == "thinbottom":
-        for x in range(xrange[0], xrange[1] + 1):
-            worksheet.cell(row = yrange[1], column = x).style.borders.bottom.border_style = openpyxl.style.Border.BORDER_THIN
-    else:
+    cell_style = None
+    if border_style == "thinbottom":   cell_style = openpyxl.style.Border.BORDER_THIN
+    if border_style == "thickbottom":  cell_style = openpyxl.style.Border.BORDER_THICK
+    if border_style == "doublebottom": cell_style = openpyxl.style.Border.BORDER_DOUBLE
+    if border_style == None:
         raise RenderingError("Unknown border style '%s'" % border_style)
+    for x in range(xrange[0], xrange[1] + 1):
+        worksheet.cell(row = yrange[1], column = x).style.borders.bottom.border_style = border_style
 
 def render(workbook, cursor, y_range, x_range, render_state, tree):
     for node in tree:
@@ -181,7 +181,7 @@ def render(workbook, cursor, y_range, x_range, render_state, tree):
         if debugging: print "Process ", node
         node_type = node['type']
         if node_type == 'attr':
-            cell = current_sheet.cell(row = cursor[0] + y_range, column = cursor[1])
+            cell = current_sheet.cell(row = cursor[0] + y_range - 1, column = cursor[1])
             caption = node.get('caption')
             if caption == None: caption = node['select']
             set_cell_value_and_wrap_if_needed(cell, caption)
@@ -193,14 +193,16 @@ def render(workbook, cursor, y_range, x_range, render_state, tree):
             cell = current_sheet.cell(row = cursor[0], column = cursor[1])
             caption = node['caption']
             new_cursor = [cursor[0], cursor[1]]
+            new_y_range = y_range
             if caption != None:
                 set_cell_value_and_wrap_if_needed(cell, caption)
                 set_cell_color_if_needed(cell, node.get('color'))
                 current_sheet.merge_cells(start_row = cursor[0], start_column = cursor[1], end_row = cursor[0], end_column = cursor[1] + x_range - 1)
                 set_range_border_if_needed(current_sheet, [cursor[0], cursor[0] + y_range - 1], [cursor[1], cursor[1] + x_range - 1], node.get('border'))
                 new_cursor[0] += 1
+                new_y_range -= 1
             children = node['content']
-            render(workbook, new_cursor, y_range - 1, x_range, render_state, children)
+            render(workbook, new_cursor, new_y_range, x_range, render_state, children)
             render_state['table.left']  = cursor[1]
             render_state['table.right'] = cursor[1] + x_range - 1
             render_state['table.top']   = cursor[0]
